
/* ====================================================================
   Copyright 2002-2004   Apache Software Foundation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */
        


package org.apache.poi.hslf.model;

import java.util.*;
import java.io.*;

import org.apache.poi.util.LittleEndian;
import org.apache.poi.hslf.exceptions.InvalidRecordFormatException;


/**
 * This class represents as meta sheet in a PowerPoint Document. It 
 *  allows access to the text within, if supported for this meta
 *  sheet type. It is only a very basic class for now
 *
 * @author Nick Burch
 */

public class MetaSheet
{

  private byte[] _sheet;
  private TextRun[] _runs = null;

  byte _type_t;
  byte _type_w;
  byte _type_x;
  byte _type_y;
  byte _type_z;

  // In case someone cares what the types might be
  public static final class CoreType {
  	public final static byte[] NOTES_PAGE_B = new byte[] { 0x00,0x0b,0x00 };
  	public final static byte[] NOTES_PAGE_C = new byte[] { 0x01,0x0c,0x00 };
  	public final static byte[] MAIN_MASTER_PAGE = new byte[] { 0x00,0x01,0x00 };
  	public final static byte[] HANDOUT_MASTER_PAGE = new byte[] { 0x03,0x06,0x02 };
  	public final static byte[] PART2_MASTER_PAGE = new byte[] { 0x01,0x02,0x00 };
  	public final static byte[] SUBTITLE_MASTER_PAGE = new byte[] { 0x01,0x04,0x02 };
  }

  /**
   * Constructs a MetaSheet from the given section of the underlying
   * powerpoint file. Copies useful things into data structures,
   * and initalises any TextRuns if supported for this type
   *
   * @param bytes the byte array to get data from
   * @param start the point in the byte array to start at
   * @param length how far into the byte array to read
   */
  public MetaSheet(byte[] bytes, int start, int length)
  {
	_sheet = new byte[length];
	System.arraycopy(bytes,start,_sheet,0,length);

	// Grab the sheet type
	_type_t = _sheet[7];
	_type_w = _sheet[11];
	_type_x = _sheet[12];
	_type_y = _sheet[13];
	_type_z = _sheet[14];

	//System.out.println("\tSheet type is " + _type_t + " : " + _type_w + " " + _type_x);

	if(_type_t == 0x00 && _type_w == 0x0b && _type_x == 0x00) {
		// Notes Page type B

		// Find the text runs 
		// (Text runs occur no earlier than 24 bytes in)
		_runs = findTextRuns(_sheet,24,length);
	}
	if(_type_t == 0x01 && _type_w == 0x0c && _type_x == 0x00) {
		// Notes Page type C

		// Find the text runs 
		// (Text runs occur no earlier than 24 bytes in)
		_runs = findTextRuns(_sheet,28,length);
	}
  }


  /**
   * Scans through the supplied byte array, looking for 
   * TextRuns. If it finds any, it initalises them.
   *
   * @@param b the byte array to get data from
   * @@param start the point in the byte array to start at
   * @@param length how far into the byte array to read
   */
  protected static TextRun[] findTextRuns(byte[] b, int start, int length) {
	Vector runs = new Vector(10);

	// Look for text runs
	// Really quite a crude way for now
	for(int i=0; i < (length - 10); i++) {
		if(b[i] == 0x00 &&
		   b[i+1] == (0x9f-256) &&
		   b[i+2] == 0x0f &&
		   b[i+3] == 0x04 &&
		   b[i+4] == 0x00 &&
		   b[i+5] == 0x00 &&
		   b[i+6] == 0x00) {
			if((length-i) > 12 ) {
				Integer startsAt = new Integer(i);
				runs.add(startsAt);
			} else {
				// End text run, doesn't contain any entries
			}
		}
	}

	// Chuck on end of our array as last end position
	runs.add(new Integer(length));

	// Loop over TextRun positions, creating
	TextRun[] foundRuns = new TextRun[runs.size()-1];
	for(int i=0; i < (runs.size()-1); i++) {
		int spos = ( (Integer)runs.get(i) ).intValue();
		int epos = ( (Integer)runs.get((i+1)) ).intValue();
		try {
			foundRuns[i] = new oldTextRun(b,spos,(epos-spos));
		} catch(InvalidRecordFormatException irfe) {
			System.out.println("Text Run between " + spos + " and " + epos + " wasn't valid: " + irfe);
		}
	}

	return foundRuns;
  }



  // Accesser methods follow

  /**
   * Returns an array of all the TextRuns found
   */
  public TextRun[] getTextRuns() { return _runs; }

  /**
   * Returns the t type.
   * @see org.apache.poi.hslf.model.readFIB()
   */
  public byte getTType() { return _type_t; }

  /**
   * Returns the w type.
   * @see org.apache.poi.hslf.model.readFIB()
   */
  public byte getWType() { return _type_w; }

  /**
   * Returns the x type.
   * @see org.apache.poi.hslf.model.readFIB()
   */
  public byte getXType() { return _type_x; }

  /**
   * Returns the y type.
   * @see org.apache.poi.hslf.model.readFIB()
   */
  public byte getYType() { return _type_y; }

  /**
   * Returns the z type.
   * @see org.apache.poi.hslf.model.readFIB()
   */
  public byte getZType() { return _type_z; }
} 
