
/* ====================================================================
   Copyright 2002-2004   Apache Software Foundation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */
        


package org.apache.poi.hslf.model;

import java.util.*;

import org.apache.poi.hslf.record.*;
import org.apache.poi.hslf.record.SlideListWithText.*;
import org.apache.poi.util.LittleEndian;

/**
 * This class represents as sheet in a PowerPoint Document. It allows 
 *  access to the text within, and the layout. For now, it only does
 *  the text side of things though
 *
 * @author Nick Burch
 */

public class Sheet
{

  private int _sheetNo;
  private SlideAtomsSet _atomSet;
  private SlidePersistAtom _slidePersistAtom;
  private TextRun[] _runs;

  /**
   * Constructs a Sheet from the given set of Atoms related to the
   *  text of the sheet. Doesn't care about the layout stuff as yet.
   * Initialises TextRuns, to provide easier access to the text
   *
   * @param atomSet the SlideAtomsSet to get the text from
   */
  public Sheet(SlideAtomsSet atomSet) {
	_atomSet = atomSet;
	_slidePersistAtom = _atomSet.getSlidePersistAtom();

	// Grab the sheet number
	_sheetNo = _slidePersistAtom.getRefID();

	// Now, build up TextRuns from pairs of TextHeaderAtom and
	//  one of TextBytesAtom or TextCharsAtom
    _runs = findTextRuns(_atomSet.getSlideRecords());
  }


  /**
   * Scans through the supplied record array, looking for 
   * a TextHeaderAtom followed by one of a TextBytesAtom or
   * a TextCharsAtom. Builds up TextRuns from these
   *
   * @param records the records to build from
   */
  protected static TextRun[] findTextRuns(Record[] records) {
	Vector runs = new Vector(10);

	// Look for a TextHeaderAtom
	for(int i=0; i<(records.length-1); i++) {
		if(records[i] instanceof TextHeaderAtom) {
			TextRun trun = null;
			TextHeaderAtom tha = (TextHeaderAtom)records[i];
			if(records[i+1] instanceof TextCharsAtom) {
				TextCharsAtom tca = (TextCharsAtom)records[i+1];
				trun = new TextRun(tha,tca);
			} else if(records[i+1] instanceof TextBytesAtom) {
				TextBytesAtom tba = (TextBytesAtom)records[i+1];
				trun = new TextRun(tha,tba);
			} else {
				System.err.println("Found a TextHeaderAtom not followed by a TextBytesAtom or TextCharsAtom: Followed by " + records[i+1].getRecordType());
				continue;
			}
			runs.add(trun);
			i++;
		}
	}

	TextRun[] foundRuns = new TextRun[runs.size()];
	for(int i=0; i<foundRuns.length; i++) {
		foundRuns[i] = (TextRun)runs.get(i);
	}

	return foundRuns;
  }


  // Accesser methods follow

  /**
   * Returns an array of all the TextRuns found
   */
  public TextRun[] getTextRuns() { return _runs; }

  /**
   * Returns the sheet number
   */
  public int getSheetNumber() { return _sheetNo; }
} 
