/* ====================================================================
   Copyright 2002-2004   Apache Software Foundation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */
        


package org.apache.poi.hslf.model;

import java.util.*;
import java.io.*;

import org.apache.poi.util.LittleEndian;
import org.apache.poi.hslf.exceptions.InvalidRecordFormatException;
import org.apache.poi.hslf.record.*;
import org.apache.poi.hslf.util.TextMunger;

/**
 * Eventually, Notes sheets will grok the Atoms within their PDDrawing
 *  atom. At that point, they can properly use TextRun.
 * Until then, we need this rather nasty version which works by scanning
 *  the byte stream looking for interesting bits, and working from that
 *
 * @@author Nick Burch
 */

public class oldTextRun extends TextRun {
  private byte[] _run;
  private boolean _isUnicode;
  private String _rawText;
  private String _text;
  private byte _runType;

  /**
   * Constructs a Text Run from a section of an underlying PDDrawing
   *  in a meta sheet.
   * Extracts the text into a string, handling unicode stuff if needed
   *
   * @@param bytes the byte array to get data from
   * @@param start the point in the byte array to start at
   * @@param length how far into the byte array to read
   */
  public oldTextRun(byte[] bytes, int start, int length) throws InvalidRecordFormatException
  {
	super((TextHeaderAtom)null,(TextCharsAtom)null);

	// If it isn't long enough, barf
	if(length < 16 || bytes.length < 16) {
		throw new InvalidRecordFormatException("Text runs need to be at least 16 bytes long, and this wasn't");
	}

	_run = new byte[length];
	System.arraycopy(bytes,start,_run,0,length);

	// Zip forward, and see what it is:
	//  00 00 9f 0f 04 00 00 00 ## 00 00 00 
	//  00 00 [A0 | A8] 0f (len len) 00 00 text text
	//
	// the contents of ## dictate what sort of text it is,
	//  eg body, notes, centre title etc. 
	//  See TextHeaderAtom (3999) for full details
	// If it's A0 0F then that's 4000 -> TextCharsAtom -> unicode
	// If it's A8 0F then that's 4008 -> TextBytesAtom -> not unicode
	//  (still need to figure out exactly which character set this is)
	_runType = _run[7];
	if(_run[13] == (0xa0 - 256)) {
		// unicode (character) text
		_isUnicode = true;
	} else if(_run[13] == (0xa8 - 256)) {
		// byte text
		_isUnicode = false;
	} else {
		// who knows
		System.err.println("Hit unknown text text, run type is " + _runType + ", text type is " + _run[13]);
		_isUnicode = false;
	}

	// Figure out how long the text is
	long size = LittleEndian.getUInt(_run,15);

	// Create the text from the bytes
	if(_isUnicode) {
		//System.out.println("Found unicode text");
		_rawText = TextMunger.pptUnicodeMakeString(_run,19,(int)size);
	} else {
		_rawText = new String(_run,19,(int)size);
	}

	// PowerPoint seems to store files with \r as the line break
	// The messes things up on everything but a Mac, so translate
	//  them to \n
	_text = _rawText.replace('\r','\n');
  }

  /** 
   * We don't support setting the text, as we don't have a proper link
   *  to the record layer. This will throw a Runtime Exception!
   */
  public void setText(String s) {
	throw new RuntimeException("Unable to set the text on TextRuns derived from PDDrawings, as there isn't proper record level support yet.");
  }


  // Accesser methods follow

  /**
   * Returns the text content of the run
   */
  public String getText() { return _text; }

  /**
   * Returns the raw text content of the run. This hasn't had any
   *  changes applied to it, and so is probably unlikely to print
   *  out nicely.
   */
  public String getRawText() { return _rawText; }

  /**
   * Returns the raw type of the run, from offset 7
   * (This corresponds to the 4 byte type from a TextHeaderAtom, we hope)
   */
  public int getRunType() { return (int)_runType; }

} 
